@@ -1,10 +1,8 @@
-import contextlib
 import contextvars
 import dataclasses
 from typing import (
     Any,
     Callable,
-    ClassVar,
     Dict,
     List,
     Optional,
@@ -165,8 +163,9 @@ def _get_model_hints(
                     store.prefetch_related.append(model_fieldname)
                 elif len(f_types) == 1:
                     remote_field = model_field.remote_field
+                    remote_model = remote_field.model
                     f_store = _get_model_hints(
-                        remote_field.model,
+                        remote_model,
                         schema,
                         f_types[0],
                         f_selection,
@@ -174,14 +173,18 @@ def _get_model_hints(
                         model_cache=model_cache,
                         level=level + 1,
                     )
-                    if (config is None or config.enable_only) and f_store.only:
+                    if (
+                        (config is None or config.enable_only)
+                        and f_store.only
+                        and not isinstance(remote_field, ManyToManyRel)
+                    ):
                         # If adding a reverse relation, make sure to select its pointer to us,
                         # or else this might causa a refetch from the database
-                        f_store.only.append(cast(str, resolve_model_field_name(remote_field)))
+                        f_store.only.append(remote_field.attname or remote_field.name)
 
-                    model_cache.setdefault(remote_field.model, []).append((level, f_store))
+                    model_cache.setdefault(remote_model, []).append((level, f_store))
 
-                    f_qs = f_store.apply(remote_field.model.objects.all(), config=config)
+                    f_qs = f_store.apply(remote_model.objects.all(), config=config)
                     f_prefetch = Prefetch(path, queryset=f_qs)
                     f_prefetch._optimizer_sentinel = _sentinel  # type:ignore
                     store.prefetch_related.append(f_prefetch)
@@ -459,7 +462,10 @@ class DjangoOptimizerExtension(Extension):
 
     """
 
-    _disabled: ClassVar[bool] = False
+    enabled: contextvars.ContextVar[bool] = contextvars.ContextVar(
+        "optimizer_enabled_ctx",
+        default=True,
+    )
 
     def __init__(
         self,
@@ -476,14 +482,10 @@ class DjangoOptimizerExtension(Extension):
             enable_prefetch_related=enable_prefetch_related_optimization,
         )
 
-    @classmethod
-    @contextlib.contextmanager
-    def disable(cls):
-        cls._disabled = True
-        yield
-        cls._disabled = False
-
     def on_request_start(self) -> AwaitableOrValue[None]:
+        if not self.enabled.get():
+            return
+
         optimizer.set(self)
 
     def on_request_end(self) -> AwaitableOrValue[None]:
@@ -498,8 +500,7 @@ class DjangoOptimizerExtension(Extension):
         **kwargs,
     ) -> AwaitableOrValue[Any]:
         ret = _next(root, info, *args, **kwargs)
-
-        if self._disabled:
+        if optimizer.get() is None:
             return ret
 
         if isinstance(ret, (BaseManager, QuerySet)):
@@ -517,7 +518,4 @@ class DjangoOptimizerExtension(Extension):
         *,
         store: Optional["OptimizerStore"] = None,
     ) -> QuerySet[_M]:
-        if self._disabled:
-            return qs
-
         return optimize(qs, info, config=self.config, store=store)
