@@ -3,6 +3,7 @@
 import abc
 import base64
 import dataclasses
+import math
 import sys
 from typing import (
     Any,
@@ -26,11 +27,12 @@ from typing import (
     overload,
 )
 from typing import _eval_type  # type:ignore
+import uuid
 
 from graphql import GraphQLID
 import strawberry
 from strawberry.annotation import StrawberryAnnotation
-from strawberry.arguments import UNSET, StrawberryArgument, is_unset
+from strawberry.arguments import UNSET, StrawberryArgument
 from strawberry.custom_scalar import ScalarDefinition
 from strawberry.field import StrawberryField
 from strawberry.permission import BasePermission
@@ -41,7 +43,9 @@ from strawberry.types.fields.resolver import StrawberryResolver
 from strawberry.types.types import TypeDefinition
 from strawberry.utils.await_maybe import AwaitableOrValue
 from strawberry.utils.str_converters import to_camel_case
+from typing_extensions import Annotated
 
+from .settings import config
 from .utils import aio
 
 __all__ = [
@@ -217,14 +221,36 @@ class GlobalID:
         return origin
 
     @overload
-    def resolve_node(self, info: Info, ensure_type: NodeType) -> AwaitableOrValue[NodeType]:
+    def resolve_node(
+        self,
+        info: Info,
+        *,
+        required: Literal[True] = ...,
+        ensure_type: Type[NodeType],
+    ) -> AwaitableOrValue[NodeType]:
         ...
 
     @overload
-    def resolve_node(self, info: Info, ensure_type=None) -> AwaitableOrValue["Node"]:
+    def resolve_node(
+        self,
+        info: Info,
+        *,
+        required: Literal[True],
+        ensure_type: None = ...,
+    ) -> AwaitableOrValue["Node"]:
         ...
 
-    def resolve_node(self, info, ensure_type=None):
+    @overload
+    def resolve_node(
+        self,
+        info: Info,
+        *,
+        required: bool = ...,
+        ensure_type: None = ...,
+    ) -> AwaitableOrValue[Optional["Node"]]:
+        ...
+
+    def resolve_node(self, info, *, required: bool = False, ensure_type: type = None):
         """Resolve the type name and node id info to the node itself.
 
         Tip: When you know the expected type, calling `ensure_type` should help
@@ -235,6 +261,9 @@ class GlobalID:
         Args:
             info:
                 The strawberry execution info resolve the type name from
+            required:
+                If the value is required to exist. Note that asking to ensure
+                the type automatically makes required true.
             ensure_type:
                 Optionally check if the returned node is really an instance
                 of this type.
@@ -248,9 +277,13 @@ class GlobalID:
 
         """
         n_type = self.resolve_type(info)
-        node = n_type.resolve_node(self.node_id, info=info)
+        node = n_type.resolve_node(
+            self.node_id,
+            info=info,
+            required=required or ensure_type is not None,
+        )
 
-        if ensure_type is not None:
+        if node is not None and ensure_type is not None:
             return aio.resolve(node, lambda n: n, info=info, ensure_type=ensure_type)
 
         return node
@@ -296,9 +329,12 @@ class Node(abc.ABC):
         type_name = info.path.typename
         assert type_name
 
-        # str is the default and is faster to check for it than is_awaitable
         if isinstance(node_id, str):
+            # str is the default and is faster to check for it than is_awaitable
             return GlobalID(type_name=type_name, node_id=node_id)
+        elif isinstance(node_id, (int, uuid.UUID)):
+            # those are very common ids and are safe to convert to str
+            return GlobalID(type_name=type_name, node_id=str(node_id))
         elif aio.is_awaitable(node_id, info=info):
             return aio.resolve_async(  # type:ignore
                 node_id,
@@ -435,8 +471,8 @@ class Node(abc.ABC):
         node_id: str,
         *,
         info: Optional[Info] = None,
-        required: Literal[False] = ...,
-    ) -> AwaitableOrValue[Optional[NodeType]]:
+        required: Literal[True],
+    ) -> AwaitableOrValue[NodeType]:
         ...
 
     @overload
@@ -447,8 +483,8 @@ class Node(abc.ABC):
         node_id: str,
         *,
         info: Optional[Info] = None,
-        required: Literal[True] = ...,
-    ) -> AwaitableOrValue[NodeType]:
+        required: bool = ...,
+    ) -> AwaitableOrValue[Optional[NodeType]]:
         ...
 
     @classmethod
@@ -611,23 +647,39 @@ class Connection(Generic[NodeType]):
         if after:
             after_type, after_parsed = from_base64(after)
             assert after_type == _connection_typename
-            start = max(start, int(after_parsed))
+            start = max(start, int(after_parsed) + 1)
         if before:
             before_type, before_parsed = from_base64(before)
             assert before_type == _connection_typename
             end = min(end, int(before_parsed))
 
+        has_first_or_last = False
+        max_results = config.RELAY_MAX_RESULTS
+        if max_results is None:
+            max_results = math.inf
+
         if isinstance(first, int):
             if first < 0:
                 raise ValueError("Argument 'first' must be a non-negative integer.")
 
+            has_first_or_last = True
+            if first > max_results:
+                raise ValueError(f"Argument 'first' cannot be higher than {max_results}.")
+
             end = min(end, start + first)
         if isinstance(last, int):
             if last < 0:
                 raise ValueError("Argument 'last' must be a non-negative integer.")
 
+            if last > max_results:
+                raise ValueError(f"Argument 'last' cannot be higher than {max_results}.")
+
+            has_first_or_last = True
             start = max(start, end - last)
 
+        if not has_first_or_last:
+            end = min(end, start + max_results)
+
         edges = [
             Edge(cursor=to_base64(_connection_typename, start + i), node=v)
             for i, v in enumerate(cast(Sequence, nodes)[start:end])
@@ -652,6 +704,7 @@ class RelayField(StrawberryField):
     default_args: Dict[str, StrawberryArgument]
 
     def __init__(self, *args, **kwargs):
+        self.default_args = getattr(self.__class__, "default_args", {}).copy()
         base_resolver = kwargs.pop("base_resolver", None)
         super().__init__(*args, **kwargs)
         if base_resolver:
@@ -775,7 +828,7 @@ class ConnectionField(RelayField):
             # If base_resolver is not self.conn_resolver, then it is defined to something
             assert self.base_resolver
 
-            resolver_args = {arg.python_name for arg in self.base_resolver.arguments}  # type:ignore
+            resolver_args = {arg.python_name for arg in self.base_resolver.arguments}
             resolver_kwargs = {k: v for k, v in kwargs.items() if k in resolver_args}
             # This will be passed to the field cconnection resolver
             kwargs = {k: v for k, v in kwargs.items() if k in self.default_args}
@@ -800,17 +853,34 @@ class InputMutationField(RelayField):
         name = to_camel_case(resolver.__name__)
         cap_name = name[0].upper() + name[1:]
         namespace = sys.modules[resolver.__module__].__dict__
+        annotations = resolver.__annotations__
         resolver = StrawberryResolver(resolver)
 
-        args = cast(List[StrawberryArgument], resolver.arguments)
+        args = resolver.arguments
         type_dict = {
             "__doc__": f"Input data for `{name}` mutation",
             "__annotations__": {},
         }
         for arg in args:
+            annotation = annotations[arg.python_name]
+            if get_origin(annotation) is Annotated:
+                directives = tuple(
+                    d for d in get_args(annotation)[1:] if isinstance(d, StrawberrySchemaDirective)
+                )
+            else:
+                directives = ()
+
             type_dict["__annotations__"][arg.python_name] = arg.type
-            if not is_unset(arg.default):
-                type_dict[arg.python_name] = arg.default
+            arg_field = strawberry.field(
+                name=arg.graphql_name,
+                is_subscription=arg.is_subscription,
+                description=arg.description,
+                default=arg.default,
+                directives=directives,
+            )
+            arg_field.graphql_name = arg.graphql_name
+            arg_field.type_annotation = arg.type_annotation
+            type_dict[arg.python_name] = arg_field
 
         # TODO: We are not creating a type for the output payload, as it is not easy to
         # do that with the typing system. Is there a way to solve that automatically?
